// A class template for a min. binary heap.
#ifndef BINARYHEAP_H
#define BINARYHEAP_H

#include <iostream>
#include <cstdlib>
#include <cassert>
using namespace std;


template <class T>
class BinaryHeap {
	static const int DEFAULT_MAX_SIZE = 100;

private:
	int maxSize;
	int numItems;
	T * heap;

	void siftUp(int currentPosition);
	void siftDown(int currentPosition);

public:
	BinaryHeap();
	BinaryHeap(int heapSize);
	BinaryHeap(const BinaryHeap & aHeap);
	void insert(const T& newItem);
	const T& findMin() const;
	T delMin();
	bool isEmpty();
	bool isFull();
	int size();
	void displayHeap();
	friend ostream& operator<< <T>(ostream & out, const BinaryHeap<T> & aHeap);
};  // end BinaryHeap class

//**************************************************
// Echos the BinaryHeap to the ostream object out  *
// in the order of the array heap.  A new-line     *
// character separates each item.                  *
//**************************************************
template <class T>
ostream& operator<<(ostream & out, const BinaryHeap<T> & aHeap) {
	for (int i = 0; i < aHeap.numItems; i++) {
		out << aHeap.heap[i] << "\n";
	} // end for
	out << endl;
	return out;
} // end operator<<

//**************************************************
// Displays BinaryHeap to cout in the order of the *
// array heap.                                     *
//**************************************************
template <class T>
void BinaryHeap<T>::displayHeap() {
	cout << "Heap: ";
	for (int i = 0; i < numItems; i++) {
		cout << heap[i] << " ";
	} // end for
	cout << endl;
} // end displayHeap

//**************************************************
// BinaryHeap default constructor which constructs *
// an empty heap with a maximum size of the        *
// DEFAULT_MAX_SIZE                                *
//**************************************************
template <class T>
BinaryHeap<T>::BinaryHeap() {
	maxSize = DEFAULT_MAX_SIZE;
	numItems = 0;
	heap = new T [DEFAULT_MAX_SIZE];
} // end default constructor

//**************************************************
// BinaryHeap constructor which constructs         *
// an empty heap with a maximum size specified by  *
// the heapSize parameter.                         *
//**************************************************
template <class T>
BinaryHeap<T>::BinaryHeap(int heapSize) {
	maxSize = heapSize;
	numItems = 0;
	heap = new T [heapSize];
} // end constructor

//**************************************************
// BinaryHeap copy constructor which constructs    *
// a new heap from the aHeap parameter.            *
//**************************************************
template <class T>
BinaryHeap<T>::BinaryHeap(const BinaryHeap & aHeap) {
	maxSize = aHeap.size;
	numItems = aHeap.numItems;
	heap = new T [maxSize];
	// copy aHeap's content
	for (int index = 0; index < numItems; index++) {
		heap[index] = aHeap[index];
	} // end for
} // end copy constructor

template <class T>
void BinaryHeap<T>::siftUp(int currentPosition) {

	// ADD CODE HERE

} // end siftUp


template <class T>
void BinaryHeap<T>::siftDown(int currentPosition) {

	//	ADD CODE HERE

} // end siftDown

//**************************************************
// Inserts newItem into BinaryHeap                 *
// Precondition:  the BinaryHeap is not full       *
// Postcondition:  the BinaryHeap contains all of  *
// its original items with newItem added.          *
//**************************************************
template <class T>
void BinaryHeap<T>::insert(const T& newItem) {

	// ADD CODE HERE


} // end insert

//**************************************************
// Returns the minimum item in the BinaryHeap      *
// without removing it, so all items remain in the *
// BinaryHeap.                                     *
// Precondition:  the BinaryHeap is not empty      *
// Postcondition:  the minimum item in the original*
// BinaryHeap is returned with all items remaining *
// in the BinaryHeap.                              *
//**************************************************
template <class T>
const T& BinaryHeap<T>::findMin() const {
	assert(numItems > 0);
	return heap[0];
} // end findMin

//**************************************************
// Removes and returns the minimum item in the     *
// BinaryHeap.  All other items remain in the      *
// BinaryHeap.                                     *
// Precondition:  the BinaryHeap is not empty      *
// Postcondition:  the minimum item in the original*
// BinaryHeap is returned with all other items     *
// remaining in the BinaryHeap.                    *
//**************************************************
template <class T>
T BinaryHeap<T>::delMin() {

	// ADD CODE HERE

} // end delMin

//**************************************************
// Returns true if the BinaryHeap contains no      *
// items; otherwise it returns false.              *
// Precondition:  none                             *
// Postcondition:  Returns true if the BinaryHeap  *
// contains no items; otherwise it returns false.  *
//**************************************************
template <class T>
bool BinaryHeap<T>::isEmpty() {
	return numItems == 0;
} // end isEmpty

//**************************************************
// Returns true if the BinaryHeap can contain no   *
// more items; otherwise it returns false.         *
// Precondition:  none                             *
// Postcondition:  Returns true if the BinaryHeap  *
// can contain no more items; otherwise it returns *
// false.                                          *
//**************************************************
template <class T>
bool BinaryHeap<T>::isFull() {
	return numItems == maxSize;
} // end isEmpty

//**************************************************
// Returns a count of the number of items in the   *
// BinaryHeap.                                     *
// Precondition:  none                             *
// Postcondition:  Returns a count of the number   *
// of items in the BinaryHeap.                     *
//**************************************************
template <class T>
int BinaryHeap<T>::size() {
	return numItems;
} // end size

#endif